; Shepard Tones as Csound Events
; Iain McCurdy, 2015.

; kRate         -       rate of note generation         
; kCentre       -       centre of texture (as a note number)
; iLayers       -       number of simultaneous voices. Should be an integer.
; kStep         -       scale step size. '1' for a rising chromatic scale. Negative values for descending scales.
; kIntvl        -       interval between adjacent layers (in semitones)
; kDur          -       duration of the note events
; kRndWhen      -       random offset of note start times (in seconds)
; iInsNum       -       instrument number of the instrument to be called 
; giAmpFn       -       amplitude scaling function applied across the complete texture 

<CsoundSynthesizer>

<CsOptions>
-odac -dm0
</CsOptions>

<CsInstruments>

0dbfs      =         1

giAmpFn    ftgen     0,0,1024,7,0,512,1,512,0

opcode  EndlessScale,0,kkkkkkkiiip
 kRate,kCentre,kStep,kIntvl,kDur,kRndWhen,kArp,iInsNum,iAmpFn,iLayers,iCount        xin
 ktrig     metro     kRate
 
 kRange    =         iLayers * kIntvl                      ; range of entire texture
 kBase     =         kCentre - (int(iLayers/2) * kIntvl)   ; base note (number) of texture
 kNote     init      i(kBase) + ((iCount-1)*i(kIntvl))     ; initial note
 
 if ktrig==1 then
  kAmp     table     (kNote-kBase)/(iLayers*kIntvl),iAmpFn,1 ; read amplitude from spectral envelope
           event     "i",iInsNum,rnd(kRndWhen)+((iCount-1)*kArp),kDur,kNote,kAmp,iCount ; call note
  kNote    wrap      kNote+kStep,kBase,kBase+(iLayers*kIntvl)                           ; increment note
 endif

 if iCount<iLayers then  ; call next layer (if required)
           EndlessScale  kRate,kCentre,kStep,kIntvl,kDur,kRndWhen,kArp,iInsNum,iAmpFn,iLayers,iCount+1 
 endif
 
endop

instr      1
 kRate     =         p4      ; rate of notes (per second)
 ktrig     metro     kRate
 kCentre   init      p5      ; centre note (note number)
 kStep     init      p6      ; step size (semitones)
 kIntvl    init      p7      ; interval between layers
 iLayers   init      p8      ; number of layers
 iInsNum   =         3       ; instrument number of instr to play a sound
 kDur      =         3/kRate ; note duration
 kRndWhen  =         p9      ; range of random note start time
 kArp      =         p10     ; arpeggiation time
           EndlessScale kRate,kCentre,kStep,kIntvl,kDur,kRndWhen,kArp,iInsNum,giAmpFn,iLayers 
endin

instr      3 ; a pluck sound
 icps      =         cpsmidinn(p4)
 iamp      =         p5
 aenv      expsegr           1,0.1,0.001
 aImpEnv   expseg    0.0001,0.001,1,10/icps,0.0001,1,0.0001
 axcite    butlp     pinkish:a(aImpEnv*iamp),limit:i(icps*2,0,sr/2)
 a1        wguide1   axcite,icps,limit:i(icps*16,0,sr/2),0.999
 a1        dcblock2  a1
           outs      a1*aenv,a1*aenv
endin

</CsInstruments>

<CsScore>
;-----------p4-----p5-----p6-----p7-----p8------p9------p10
;----------rate--centre--step--intvl--layers--RndWhen--Arp
i 1 0  14   2      60     1     12       8       0       0
i 1 15 14   2      60    0.5    12       8       0       0
i 1 30 14   4      48     2      5      12       0     [1/3]
i 1 47 14   3      36    -1      7      12     0.1       0
</CsScore>

</CsoundSynthesizer>