; Shepard Tones with MIDI Input
; Iain McCurdy, 2015.

; iCentre         -       centre of texture (as a note number)
; iLayers         -       number of simultaneous voices. Should be an integer.
; iIntvl          -       interval between adjacent layers (in semitones)
; iInsNum         -       instrument number of the instrument to be called 
; giAmpFn         -       amplitude scaling function applied across the complete texture 

<CsoundSynthesizer>

<CsOptions>
-odac -dm0 -Ma
</CsOptions>

<CsInstruments>

0dbfs      =         1

giAmpFn    ftgen     0,0,1024,7,0,512,1,512,0

opcode  EndlessScaleMIDI,a,iiiiiiiip
 iNoteNum,iStepSize,iCentre,iVel,iLayers,iIntvl,iInsNum,iAmpFn,iCount        xin
 iStep     =         (iNoteNum*iStepSize)%iIntvl           ; wrapped step value
 iRange    =         iLayers * iIntvl                      ; range of entire texture
 iBase     =         iCentre - (int(iLayers/2) * iIntvl)   ; base (lowest) note (number) of texture
 iNote     wrap      iBase + ((iCount-1)*iIntvl) + iStep, iBase, iBase+(iLayers*iIntvl) ; note (number) for this layer

 iAmp     table      (iNote-iBase)/(iLayers*iIntvl),iAmpFn,1
 aSig     subinstr   iInsNum,iNote,iAmp*iVel,iCount
 if iCount<iLayers then
  aMix    EndlessScaleMIDI  iNoteNum,iStepSize,iCentre,iVel,iLayers,iIntvl,iInsNum,iAmpFn,iCount+1 
 endif
          xout       aSig + aMix
endop

instr      1
 iNoteNum  notnum            ; MIDI note number
 iStepSize =         0.5     ; step size from note to note
 iIntvl    =         7       ; interval between layers
 iCentre   =         36      ; centre note (note number)
 iVel      veloc     0,1     ; MIDI note velocity
 iLayers   =         12      ; number of layers
 iInsNum   =         3       ; instrument number of instr to play a sound
 aMix      EndlessScaleMIDI iNoteNum,iStepSize,iCentre,iVel,iLayers,iIntvl,iInsNum,giAmpFn 
 aEnv      expsegr   1,0.1,0.001 ; anti click envelope
           out       aMix*aEnv
endin

instr      3 ; a pluck sound
 icps      =         cpsmidinn(p4)
 iAmp      =         p5
 aImpEnv   expseg    0.0001,0.001,1,10/icps,0.0001,1,0.0001
 axcite    butlp     pinkish:a(aImpEnv*iAmp),limit:i(icps*2,0,sr/2)
 aSig      wguide1   axcite,icps,limit:i(icps*16,0,sr/2),0.999
 aSig      dcblock2  aSig
           out       aSig
endin

</CsInstruments>

</CsoundSynthesizer>