; Endless Speed Change
; Iain McCurdy 2015

; kInsNum	-	number of the sound producing instrument that will be called from the UDO
; kDur		-	duration of note events.
;			if a negative value is provided, the absolute value is used as the duration but if a new event will overlap with the previous note from that layer,
;			then the older note will be stopped. This is a kind of per-layer monophonic polyphony control intended to conserve resources when note generation rates are high.
; iLayers	-	number of layers employed by the texture
; kMult		-	value that will be multiplied to the rate of each subsequent layer. 
;			For example, if kMult=2, layer 2 will be twice that rate of layer 1 and layer 3 will be four times the rate of layer 1.
; kCentre       -       centre of texture (as a note number)
; kPhase        -       global control of the phase of the gliss. Should be a moving value within the range 0 to 1.

<CsoundSynthesizer>

<CsOptions>
-odac -dm0
</CsOptions>

<CsInstruments>

0dbfs     =         1                                                                                                                 

giAmpFn  ftgen 0,0,1024,7,0,512,1,512,0 ; triangle

opcode  EndlessSpeedChange,0,kkkkkiip
 kInsNum,kDur,kPhase,kCentre,kMult,iAmpFn,iLayers,iCount  xin
 kLPhs      wrap     kPhase+((iCount-1)/iLayers),0,1 ; local phase
 kAmp       tablei   kLPhs,iAmpFn,1                  ; read amplitude value from envelope
 kRate      scale    kLPhs,iLayers/2,-iLayers/2      ; rate function (in octaves)
 kRate      =        kCentre * (kMult ^ kRate)       ; rate (hz) 
 kTrig      metro    kRate                           ; trigger for this layer

  if kDur<=0 then                                    ; event duration control if negative value
   kDur2            limit       abs(kDur),0.1,1/kRate ; limit to time gap (1/kRate) 
  else
   kDur2            =           kDur                 ; otherwise just use duration
  endif
  
          schedkwhen  kTrig,0,0,kInsNum,0,kDur2,kAmp,iCount
 if iCount<iLayers then                              ; call next layer if required
     EndlessSpeedChange kInsNum,kDur,kPhase,kCentre,kMult,iAmpFn,iLayers,iCount+1
 endif
endop

instr      1 ; define texture values and call UDO
 kAmp      =         0.2
 kInsNum   init      3       ; sound producing instrument that will be called by events 
 kDur      init      -1.3    ; negative duration activates duration truncation mechanism for polyphony control
 iLayers   =         4       ; number of layers
 kMult      init      2      ; rate multiplier
 kCentre   init      4       ; centre frequency (rate)
 kPhase    phasor    -0.1    ; phase position through entire pattern
           EndlessSpeedChange  kInsNum,kDur,kPhase,kCentre,kMult,giAmpFn,iLayers
endin

instr      3 ; sound producing instrument
 iLayer    =         p5
 iNote     =         48 + (iLayer * 5)
 iDur      =         0.9*(1+(((64-iNote)/127)*3))
 iVel      =         p4^3
 aEnv1     expsegr   1,     iDur,0.0001,0.01,0.0001
 aEnv2     expsegr   1,0.2* iDur,0.0001,0.01,0.0001
 aEnv3     expsegr   1,0.02*iDur,0.0001,0.01,0.0001
 aPart1    oscili    (aEnv1-0.0001) * 1   , cpsmidinn(iNote)
 aPart2    oscili    (aEnv2-0.0001) * iVel, cpsmidinn(iNote) * 3.37
 aMod      oscili    aEnv3,cpsmidinn(iNote) * 2.77
 aMix      sum       (aPart1+aPart2)*iVel*0.3*(1-(aMod*iVel)) 
           out       aMix 
endin

</CsInstruments>

<CsScore>
i 1 0 300
</CsScore>

</CsoundSynthesizer>
