#
#   Flask server2, showing websockets (bi-directional) communication
#
from gevent import monkey
monkey.patch_all()

import time
import csnd6 as csnd
from threading import Thread
from flask import Flask, render_template, session, request
from flask.ext.socketio import SocketIO, emit, join_room, leave_room, close_room, disconnect

app = Flask(__name__, static_url_path='')
app.debug = False

app.config['SECRET_KEY'] = 'secret!'
socketio = SocketIO(app)
thread = None

import logging
#   this is to suppress informational messages
log = logging.getLogger('werkzeug')
log.setLevel(logging.ERROR)

cs = csnd.Csound()
# local on OS X
res = cs.Compile('/Users/richard/Desktop/MyPython/flasky2/static/powershape.csd')
# on the Pi
#res = cs.Compile('/home/pi/Desktop/python/flasky2/static/powershape.csd')
if res == 0:
	perf = csnd.CsoundPerformanceThread(cs)
	perf.Play()   # to play

def background_thread():
	#   Poll the volume level and send to the client
	while True:
		time.sleep(.05)
		vol = cs.GetChannel("volume")
		#print vol
		socketio.emit('volume',
                      {'data': 'Server generated event', 'vol': vol},
                      namespace='/test')


@app.route('/')
def index():
    global thread
    if thread is None:
        thread = Thread(target=background_thread)
        thread.start()
    return render_template('gitarFxVU.html')


# this decorator passes the channel name and value to Csound
@socketio.on('channel', namespace='/test')
def test_message(message):
	#print message['channel'], message['data']
	cs.SetChannel(str(message['channel']), float(message['data']))


@socketio.on('disconnect request', namespace='/test')
def disconnect_request():
    emit('my response',
         {'data': 'Disconnected!'})
    disconnect()


@socketio.on('connect', namespace='/test')
def test_connect():
    emit('my response', {'data': 'Connected'})


@socketio.on('disconnect', namespace='/test')
def test_disconnect():
    print('Client disconnected')


if __name__ == '__main__':
	print ('Starting Flask server with socketio')
	socketio.run(app, host='0.0.0.0')